﻿require 'zip'
require 'fileutils'

class ZipFileGenerator

    def initialize(input_dir, output_file)
        @input_dir = input_dir
        @output_file = output_file
    end

    def write
        entries = Dir.entries(@input_dir) - %w(. ..)

        Zip.unicode_names = true
		::Zip::File.open(@output_file, ::Zip::File::CREATE) do |zipfile|
		  write_entries entries, '', zipfile
		end
    end

    private

    def write_entries(entries, path, zipfile)
        entries.each do |e|
            zipfile_path = path == '' ? e : File.join(path, e)
            disk_file_path = File.join(@input_dir, zipfile_path)
            #puts "Deflating #{disk_file_path}"

            if File.directory? disk_file_path
                recursively_deflate_directory(disk_file_path, zipfile, zipfile_path)
            else
                put_into_archive(disk_file_path, zipfile, zipfile_path)
            end
        end
    end

    def recursively_deflate_directory(disk_file_path, zipfile, zipfile_path)
        zipfile.mkdir zipfile_path.encode('cp932')
        subdir = Dir.entries(disk_file_path) - %w(. ..)
        write_entries subdir, zipfile_path, zipfile
    end

    def put_into_archive(disk_file_path, zipfile, zipfile_path)
        zipfile.get_output_stream(zipfile_path.encode('cp932')) do |f|
            #f.write(File.open(disk_file_path, 'rb').read)
            IO.copy_stream(File.open(disk_file_path, 'rb'), f)
        end
    end

end